@@ -130,7 +130,7 @@ void encode_cell(pTHX_ SV *dest, SV *src, struct cc_type *type)
 
 void encode_tinyint(pTHX_ SV *dest, SV *src)
 {
-    char bytes[5];
+    unsigned char bytes[5];
     int number;
 
     number = SvIV(src);
@@ -141,40 +141,40 @@ void encode_tinyint(pTHX_ SV *dest, SV *src)
     memset(bytes, 0, 3);
     bytes[3] = 1;
     bytes[4] = number;
-    sv_catpvn(dest, bytes, 5);
+    sv_catpvn(dest, (char*)bytes, 5);
 }
 
 void encode_smallint(pTHX_ SV *dest, SV *src)
 {
     union {
         uint16_t s[3];
-        char c[6];
+        unsigned char c[6];
     } stuff;
     memset(stuff.c, 0, 3);
     stuff.c[3] = 2;
     stuff.s[2] = htons((int16_t)SvIV(src));
-    sv_catpvn(dest, stuff.c, 6);
+    sv_catpvn(dest, (char*)stuff.c, 6);
 }
 
 void encode_int(pTHX_ SV *dest, SV *src)
 {
     union {
         uint32_t s[2];
-        char c[8];
+        unsigned char c[8];
     } stuff;
     memset(stuff.c, 0, 3);
     stuff.c[3] = 4;
     stuff.s[1] = htonl((int32_t)SvIV(src));
-    sv_catpvn(dest, stuff.c, 8);
+    sv_catpvn(dest, (char*)stuff.c, 8);
 }
 
-// XXX: this will break on some 32bit systems.
+#ifdef CAN_64BIT
 void encode_bigint(pTHX_ SV *dest, SV *src)
 {
-    char work[12];
+    unsigned char work[12];
     union {
         int64_t iv;
-        char c[8];
+        unsigned char c[8];
     } stuff;
     stuff.iv = SvIV(src);
 
@@ -184,8 +184,48 @@ void encode_bigint(pTHX_ SV *dest, SV *src)
     work[3] = 8;
     bswap8(stuff.c);
     memcpy(work+4, stuff.c, 8);
-    sv_catpvn(dest, work, 12);
+    sv_catpvn(dest, (char*)work, 12);
 }
+#else
+void encode_bigint(pTHX_ SV *dest, SV *src)
+{
+    SV *tmp_sv;
+    int sv_len;
+    char *ptr;
+    unsigned char work[12];
+
+    work[0] = 0;
+    work[1] = 0;
+    work[2] = 0;
+    work[3] = 8;
+
+    tmp_sv = sv_2mortal(newSV(8));
+    SvPOK_on(tmp_sv);
+    SvCUR_set(tmp_sv, 0);
+
+    encode_varint(aTHX_ tmp_sv, src, &sv_len);
+    if (UNLIKELY(sv_len > 8)) {
+        /* Unlike our 64bit code, we have the chance to actually detect wrapping.
+         * So if you're "lucky" and run a 32bit Perl, enjoy a warning on top of
+         * the wrapping you probably didn't want to happen. */
+        warn("Truncating scalar value: does not fit bigint");
+        sv_chop(tmp_sv, SvPV_nolen(tmp_sv)+(sv_len-8));
+        sv_len = 8;
+    }
+    assert(sv_len > 0);
+    ptr = SvPV_nolen(tmp_sv);
+    if (ptr[0] & 0x80) {
+        // Negative
+        memset(work+4, 0xff, 8);
+    } else {
+        // Positive
+        memset(work+4, 0, 8);
+    }
+    memcpy(work+4+(8-sv_len), ptr, sv_len);
+
+    sv_catpvn(dest, (char*)work, 12);
+}
+#endif
 
 void encode_blob(pTHX_ SV *dest, SV *src)
 {
@@ -202,10 +242,10 @@ void encode_blob(pTHX_ SV *dest, SV *src)
 
 void encode_float(pTHX_ SV *dest, SV *src)
 {
-    char work[8];
+    unsigned char work[8];
     union {
         float f;
-        char c[8];
+        unsigned char c[8];
     } stuff;
     stuff.f = SvNV(src);
     work[0] = 0;
@@ -214,15 +254,15 @@ void encode_float(pTHX_ SV *dest, SV *src)
     work[3] = 4;
     bswap4(stuff.c);
     memcpy(work+4, stuff.c, 4);
-    sv_catpvn(dest, work, 8);
+    sv_catpvn(dest, (char*)work, 8);
 }
 
 void encode_double(pTHX_ SV *dest, SV *src)
 {
-    char work[12];
+    unsigned char work[12];
     union {
         double d;
-        char c[8];
+        unsigned char c[8];
     } stuff;
     stuff.d = SvNV(src);
     work[0] = 0;
@@ -231,12 +271,12 @@ void encode_double(pTHX_ SV *dest, SV *src)
     work[3] = 8;
     bswap8(stuff.c);
     memcpy(work+4, stuff.c, 8);
-    sv_catpvn(dest, work, 12);
+    sv_catpvn(dest, (char*)work, 12);
 }
 
 void encode_boolean(pTHX_ SV *dest, SV *src)
 {
-    char bytes[5];
+    unsigned char bytes[5];
     memset(bytes, 0, 3);
     bytes[3] = 1;
     if (SvTRUE(src)) {
@@ -244,7 +284,7 @@ void encode_boolean(pTHX_ SV *dest, SV *src)
     } else {
         bytes[4] = 0;
     }
-    sv_catpvn(dest, bytes, 5);
+    sv_catpvn(dest, (char*)bytes, 5);
 }
 
 void encode_uuid(pTHX_ SV *dest, SV *src)
@@ -252,7 +292,7 @@ void encode_uuid(pTHX_ SV *dest, SV *src)
     char *ptr;
     STRLEN size;
     int i, j;
-    char work[20];
+    unsigned char work[20];
     memset(work, 0, 20);
 
     work[3] = 16;
@@ -281,7 +321,7 @@ void encode_uuid(pTHX_ SV *dest, SV *src)
     if (j != 32)
         warn("UUID '%s' is invalid", ptr);
 
-    sv_catpvn(dest, work, 20);
+    sv_catpvn(dest, (char*)work, 20);
 }
 
 void encode_inet(pTHX_ SV *dest, SV *src)
@@ -289,7 +329,7 @@ void encode_inet(pTHX_ SV *dest, SV *src)
     char *ptr;
     STRLEN size;
     int semicolon, i;
-    char out[20];
+    unsigned char out[20];
 
     ptr = SvPV(src, size);
     semicolon = 0;
@@ -305,7 +345,7 @@ void encode_inet(pTHX_ SV *dest, SV *src)
         out[3] = 16;
 
         if (inet_pton(AF_INET6, ptr, out+4)) {
-            sv_catpvn(dest, out, 20);
+            sv_catpvn(dest, (char*)out, 20);
         } else {
             warn("Inet address '%s' is invalid", ptr);
             encode_undef(aTHX_ dest);
@@ -314,7 +354,7 @@ void encode_inet(pTHX_ SV *dest, SV *src)
         out[3] = 4;
 
         if (inet_pton(AF_INET, ptr, out+4)) {
-            sv_catpvn(dest, out, 8);
+            sv_catpvn(dest, (char*)out, 8);
         } else {
             warn("Inet address '%s' is invalid", ptr);
             encode_undef(aTHX_ dest);
@@ -324,7 +364,7 @@ void encode_inet(pTHX_ SV *dest, SV *src)
 
 void encode_time(pTHX_ SV *dest, SV *src)
 {
-    char out[12];
+    unsigned char out[12];
     STRLEN size, i, j, k;
     char *ptr;
     int numbers[4];
@@ -357,7 +397,7 @@ void encode_time(pTHX_ SV *dest, SV *src)
     seconds = (((numbers[0]%24) * 3600) + (numbers[1] * 60) + numbers[2]) % 86400;
     *((int64_t*)(out+4)) = (((int64_t)seconds)*1000000000L) + (int64_t)nano;
     bswap8(out+4);
-    sv_catpvn(dest, out, 12);
+    sv_catpvn(dest, (char*)out, 12);
 }
 
 inline int div_properly(int a, int b)
@@ -468,13 +508,13 @@ void encode_varint(pTHX_ SV *dest, SV *src, int* int_out)
 
     } else {
         struct cc_bignum bn;
-        char *tmp, *tmp2;
+        unsigned char *tmp, *tmp2;
         size_t encoded_len, i;
 
         cc_bignum_init_string(&bn, ptr, size);
 
-        Newxz(tmp, bn.length+2, char);
-        Newxz(tmp2, bn.length+2, char);
+        Newxz(tmp, bn.length+2, unsigned char);
+        Newxz(tmp2, bn.length+2, unsigned char);
 
         encoded_len = cc_bignum_byteify(&bn, tmp, bn.length+2);
         for (i = 0; i < encoded_len; i++) {
@@ -485,7 +525,7 @@ void encode_varint(pTHX_ SV *dest, SV *src, int* int_out)
             *int_out= encoded_len;
         else
             pack_int(aTHX_ dest, encoded_len);
-        sv_catpvn(dest, tmp2, encoded_len);
+        sv_catpvn(dest, (char*)tmp2, encoded_len);
 
         Safefree(tmp);
         Safefree(tmp2);
