@@ -1,8 +1,8 @@
 package Cassandra::Client::Policy::LoadBalancing::Default;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Policy::LoadBalancing::Default::VERSION = '0.13_006'; # TRIAL
+$Cassandra::Client::Policy::LoadBalancing::Default::VERSION = '0.13_007'; # TRIAL
 
-$Cassandra::Client::Policy::LoadBalancing::Default::VERSION = '0.13006';use 5.010;
+$Cassandra::Client::Policy::LoadBalancing::Default::VERSION = '0.13007';use 5.010;
 use strict;
 use warnings;
 use List::Util 'shuffle';
@@ -72,7 +72,7 @@ sub get_next_candidate {
     return shift @$candidates;
 }
 
-my @all_backoff= (1, 5, 20, 60, 180, 600);
+my @all_backoff= map { $_ * (rand()*.4 + 0.8) } (1, 5, 20, 60, 180, 600);
 sub check_backoff {
     my ($self, $peer)= @_;
     my $times= $self->{try_times}{$peer};
@@ -123,7 +123,7 @@ Cassandra::Client::Policy::LoadBalancing::Default
 
 =head1 VERSION
 
-version 0.13_006
+version 0.13_007
 
 =head1 AUTHOR
 
